@@ -92,6 +92,8 @@ rs_runjob_alt <- function(script, name, wait = TRUE,
 #' @param rs whether to use 'RStudio' by default
 #' @param wait whether to wait for the result.
 #' @param packages packages to load in the sub-sessions
+#' @param as_promise whether to return as a \code{\link[promises]{promise}}
+#' object; default is no
 #' @param focus_on_console whether to return back to console after creating
 #' jobs; useful when users want to focus on writing code; default is false.
 #' This feature works with 'RStudio' (\code{>=1.4})
@@ -143,6 +145,40 @@ rs_runjob_alt <- function(script, name, wait = TRUE,
 #'
 #' @export
 rs_exec <- function(expr, name = 'Untitled', quoted = FALSE, rs = TRUE,
+                    as_promise = FALSE, wait = FALSE, packages = NULL,
+                    focus_on_console = FALSE, ..., nested_ok = FALSE) {
+  if(as_promise && !package_installed("promises")) {
+    stop("Cannot run `rs_exec`: Please install `promises` & `later` package")
+  }
+  if(!quoted){
+    expr <- substitute(expr)
+  }
+  check <- rs_exec_internal(
+    expr = expr, name = name, quoted = TRUE, rs = rs, wait = wait,
+    packages = packages, focus_on_console = focus_on_console,
+    nested_ok = nested_ok, ...)
+
+  if(!as_promise) {
+    return(check)
+  }
+  promises::promise(function(resolve, reject) {
+    f <- function() {
+      status <- check()
+      if(status == 0) {
+        resolve(attr(status, "rs_exec_result"))
+        return()
+      } else if (status < 0) {
+        reject(attr(status, 'rs_exec_error'))
+        return()
+      } else {
+        later::later(f, delay = 2)
+      }
+    }
+    f()
+  })
+}
+
+rs_exec_internal <- function(expr, name = 'Untitled', quoted = FALSE, rs = TRUE,
                     wait = FALSE, packages = NULL, focus_on_console = FALSE,
                     ..., nested_ok = FALSE){
   if(!nested_ok && !is_master()) {
@@ -301,248 +337,6 @@ print.dipsaus_rs_exec_res <- function(x, ...){
   invisible(x)
 }
 
-rs_install_r <- function(packages, repos = getOption('repos'),
-                        check_libpaths = TRUE, unload = NULL, rs = TRUE){
-  packages <- unique(packages)
-  unload <- unique(c('startup', unload))
-  unload <- unload[!unload %in% c('base', 'utils')]
-
-  repos <- unlist(repos, use.names = TRUE)
-  repos[['dipterix']] <- "https://dipterix.github.io/drat/"
-
-  if(isTRUE(repos[['CRAN']] == "@CRAN@")){
-    repos[['CRAN']] <- "https://cran.rstudio.com/"
-  }
-
-  if(rs){
-    rprof <- startup::find_rprofile()
-    if(length(rprof)){
-      for(f in rprof){
-        file.rename(f, paste0(f, strftime(Sys.time(), '.bak.%Y%m%d-%H%M%S')))
-      }
-    }
-  }
-
-  quo <- rlang::quo({
-
-    if(!!check_libpaths){
-      # ---- Check whether user library is created
-      vars <- c("R_LIBS", "R_LIBS_SITE", "R_LIBS_USER")
-      for (var in vars) {
-        path <- Sys.getenv(var)
-        if (!nzchar(path)) next
-        is_dummy <- grepl("^[.]", path) && !grepl("[/\\]", path)
-        if (is_dummy) next
-        paths <- unlist(strsplit(path, split = .Platform$path.sep, fixed = TRUE))
-        paths <- unique(paths)
-        paths <- paths[!vapply(paths, FUN = dir.exists, FUN.VALUE = FALSE)]
-        if ( length(paths) ) {
-          tryCatch({
-            for(p in paths){
-              dir.create(p, recursive = TRUE, showWarnings = FALSE)
-              if(dir.exists(p)){
-                .libPaths(p)
-              }
-            }
-          })
-        }
-      }
-    }
-
-    repos <- !!repos
-
-    installr <- function(...){
-      pkgs <- c(...)
-      cat('Installing ', pkgs, '\n')
-      try({
-        do.call('install.packages', list(
-          pkgs = pkgs,
-          repos = repos
-        ))
-      })
-    }
-    detect_package <- function(p){
-      system.file('', package = p) != ''
-    }
-
-    ns_reg <- function(){
-      eval(parse(text='(get(".Internal", envir = baseenv(), mode = "function"))(getNamespaceRegistry())'))
-    }
-    unregister_ns <- function (name = NULL) {
-      if (!is.character(name) || name == "" || length(name) != 1)
-        return()
-      if (!(name %in% loadedNamespaces()))
-        return()
-      do.call(rm, args = list(name, envir = ns_reg()))
-      return()
-    }
-
-    if(!!rs){
-      # Remove startup scripts
-      # install startup to avoid previous installation warnings
-      if(!detect_package('startup')){
-        # this is required by dipsaus
-        installr("startup")
-      }
-
-      tryCatch({
-        # remove startup scripts
-        startup::uninstall()
-      }, error = function(e){}, warning = function(e){})
-    }
-
-
-
-    # If reticulate is installed, Rcpp won't upgrade
-    for(ns in !!c(unload, packages)){
-      try({
-        do.call('unloadNamespace', list(ns))
-      })
-      if(ns %in% loadedNamespaces()){
-        unregister_ns(ns)
-      }
-    }
-
-    for(p in !!packages){
-      installr(p)
-    }
-
-    if(!!rs){
-      if(detect_package('startup')){
-        startup::install(overwrite = FALSE)
-      }
-    }
-
-    cat('Done\n')
-  })
-
-  rs_exec(rlang::quo_squash(quo), quoted = TRUE, name = 'Install packages', rs = rs, wait = TRUE)
-
-  invisible()
-}
-
-
-rs_install_github <- function(packages, repos = getOption('repos'),
-                        check_libpaths = TRUE, unload = NULL, rs = TRUE){
-
-  stopifnot2(package_installed('remotes'), msg = 'package `remotes` is missing. Please install it.')
-
-  packages <- unique(packages)
-  unload <- unique(c('startup', unload))
-  unload <- unload[!unload %in% c('base', 'utils')]
-
-  repos <- unlist(repos, use.names = TRUE)
-  repos[['dipterix']] <- "https://dipterix.github.io/drat/"
-
-  if(isTRUE(repos[['CRAN']] == "@CRAN@")){
-    repos[['CRAN']] <- "https://cran.rstudio.com/"
-  }
-
-  if(rs){
-    rprof <- startup::find_rprofile()
-    if(length(rprof)){
-      for(f in rprof){
-        file.rename(f, paste0(f, strftime(Sys.time(), '.bak.%Y%m%d-%H%M%S')))
-      }
-    }
-  }
-
-
-  quo <- rlang::quo({
-
-    if(!!check_libpaths){
-      # ---- Check whether user library is created
-      vars <- c("R_LIBS", "R_LIBS_SITE", "R_LIBS_USER")
-      for (var in vars) {
-        path <- Sys.getenv(var)
-        if (!nzchar(path)) next
-        is_dummy <- grepl("^[.]", path) && !grepl("[/\\]", path)
-        if (is_dummy) next
-        paths <- unlist(strsplit(path, split = .Platform$path.sep, fixed = TRUE))
-        paths <- unique(paths)
-        paths <- paths[!vapply(paths, FUN = dir.exists, FUN.VALUE = FALSE)]
-        if ( length(paths) ) {
-          tryCatch({
-            for(p in paths){
-              dir.create(p, recursive = TRUE, showWarnings = FALSE)
-              if(dir.exists(p)){
-                .libPaths(p)
-              }
-            }
-          })
-        }
-      }
-    }
-
-    repos <- !!repos
-
-    installr <- function(...){
-      pkgs <- c(...)
-      cat('Installing ', pkgs, '\n')
-      remotes::install_github(pkgs, upgrade = 'never', force = TRUE, repos = repos)
-    }
-    detect_package <- function(p){
-      system.file('', package = p) != ''
-    }
-
-    ns_reg <- function(){
-      eval(parse(text='(get(".Internal", envir = baseenv(), mode = "function"))(getNamespaceRegistry())'))
-    }
-    unregister_ns <- function (name = NULL) {
-      if (!is.character(name) || name == "" || length(name) != 1)
-        return()
-      if (!(name %in% loadedNamespaces()))
-        return()
-      do.call(rm, args = list(name, envir = ns_reg()))
-      return()
-    }
-
-    if(!!rs){
-      # Remove startup scripts
-      # install startup to avoid previous installation warnings
-      if(!detect_package('startup')){
-        # this is required by dipsaus
-        installr("startup")
-      }
-
-      tryCatch({
-        # remove startup scripts
-        startup::uninstall()
-      }, error = function(e){}, warning = function(e){})
-    }
-
-
-
-
-    # If reticulate is installed, Rcpp won't upgrade
-    for(ns in !!c(unload, packages)){
-      try({
-        do.call('unloadNamespace', list(ns))
-      })
-      if(ns %in% loadedNamespaces()){
-        unregister_ns(ns)
-      }
-    }
-
-    for(p in !!packages){
-      installr(p)
-    }
-
-    if(!!rs){
-      if(detect_package('startup')){
-        startup::install(overwrite = FALSE)
-      }
-    }
-
-    cat('Done\n')
-  })
-
-  rs_exec(rlang::quo_squash(quo), quoted = TRUE, name = 'Install packages', rs = rs, wait = TRUE)
-
-  invisible()
-}
-
-
 
 #' Get 'RStudio' active project
 #' @param ... passed to \code{\link{rs_avail}}
